package ssd;

import java.text.SimpleDateFormat;
import java.util.Date;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * TODO: Implement this content handler.
 */
public class BiddingHandler extends DefaultHandler {
	/**
	 * Use this xPath variable to create xPath expression that can be
	 * evaluated over the XML document.
	 */
	private static XPath xPath = XPathFactory.newInstance().newXPath();
	
	/**
	 * Store and manipulate the auctions XML document here.
	 */
	private Document auctionsDoc;
	
	/**
	 * This variable stores the text content of XML Elements.
	 */
	private String eleText;

	/**
	 * Insert local variables here
	 */
	private String id = "";
	private String user = "";
	private String bid = "";
	
    
	
    public BiddingHandler(Document doc) {
    	auctionsDoc = doc;
    }
    
    @Override
    /**
     * SAX calls this method to pass in character data
     */
  	public void characters(char[] text, int start, int length)
  			    throws SAXException {
  		eleText = new String(text, start, length);
  	}

    /**
     * 
     * Return the current stored auctions document
     * 
     * @return XML Document
     */
	public Document getDocument() {
		return auctionsDoc;
	}
    
    //***TODO***
	//Specify additional methods to parse the bid document and modify the auctionsDoc
	@Override
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {
    	//<bid> tag detected
    	if ("bid".equals(localName)) {
    		if (id.equals(""))
    		   id = atts.getValue("product");
    	}
    }
    
    @Override
    public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
    	try {
    		//</user> tag detected - eleText contains username
    		if ("user".equals(localName)) {
    			user = eleText;
    		}
    		//</bid> tag detected 
    		if ("bid".equals(localName)) {
    			//bid not yet set, eleText contains bid
    			if (bid.equals(""))
    				bid = eleText;
    		}
    	} catch (Exception e) {
			e.printStackTrace();
		    System.exit(1);
		}
    }
    
	@Override
	public void endDocument() throws SAXException {
		Element productNode;
		
		try {
			//First check if auction is still running
			XPathExpression xpe = xPath.compile("//products/product[@id = \"" + id + "\"]");
			NodeList productsList = (NodeList) xpe.evaluate(auctionsDoc, XPathConstants.NODESET);
			
			if (productsList.getLength() > 0) {
				Node product = productsList.item(0);
				SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
				Date auctionEnd = formatter.parse(product.getAttributes().getNamedItem("auctionEnd").getTextContent());
				
				if (auctionEnd.before(new Date())) {
					SSD.exit("Auction for Product with id = " + id + " has ended.");
				}
				
				NodeList productChildren = product.getChildNodes();
				for (int i = 0; i < productChildren.getLength(); i++) {
					if (productChildren.item(i).getNodeType() == Node.ELEMENT_NODE && productChildren.item(i).getNodeName().equals("expired"))
						SSD.exit("Auction for Product with id = " + id + " is expired.");
				}
			} else
				SSD.exit("Product not found!");
			
			
			//Select the product node with the correct id
			xpe = xPath.compile("//bids/product[@id = \"" + id + "\"]");
			NodeList bidsList = (NodeList) xpe.evaluate(auctionsDoc, XPathConstants.NODESET);
			if (bidsList.getLength() == 0) {
				productNode = auctionsDoc.createElement("product");
				productNode.setAttribute("id", id);
				
				//Insert a new gameNode in the document
				NodeList bidsEleL = auctionsDoc.getElementsByTagName("bids");
				Element bidsEle;
				if (bidsEleL.getLength() == 0) {
					bidsEle = auctionsDoc.createElement("bids");
					auctionsDoc.appendChild(bidsEle);
				} else
					bidsEle = (Element)bidsEleL.item(0);
				bidsEle.appendChild(productNode);
			} else {
				productNode = (Element) bidsList.item(0);
			}
			
			//Placed bid must be larger than the bids already placed for the product
			NodeList bids = productNode.getElementsByTagName("bid");
			for (int i = 0; i < bids.getLength(); i++) 
				if (Integer.parseInt(bids.item(i).getTextContent()) > Integer.parseInt(bid))
				   SSD.exit("Bid to low!");
			
			//Balance of user must be larger than the placed bid
			xpe = xPath.compile("//user[@username=\"" + user + "\"]/balance");
			Node balance = (Node) xpe.evaluate(auctionsDoc, XPathConstants.NODE);
			if (balance != null) {
				if (Integer.parseInt(balance.getTextContent()) < Integer.parseInt(bid))
					SSD.exit("Bid higher than balance!");
			} else {
				SSD.exit("User not found!");
			}
			
			
			Element bidEle = auctionsDoc.createElement("bid");
			bidEle.setAttribute("user", user);
			bidEle.setTextContent(bid);
			
			productNode.appendChild(bidEle);
			
					
		}
		catch (Exception ex) {
		   System.err.println(ex.getMessage());
		}
	}
    
    
	
}

